(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.rkgttrArrayfrompolyfill = factory());
}(this, (function () { 'use strict';

/**
 * rkgttr-arrayfrompolyfill
 *
 * Copyright © 2016 Erik Guittiere. All rights reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.txt file in the root directory of this source tree.
 */
var ArrayFromPolyfill = function () {
  if (!Array.from) {
    Array.from = function () {
      var toStr = Object.prototype.toString,
          isCallable = function isCallable(fn) {
        return typeof fn === 'function' || toStr.call(fn) === '[object Function]';
      },
          toInteger = function toInteger(value) {
        var number = Number(value);
        if (isNaN(number)) {
          return 0;
        }
        if (number === 0 || !isFinite(number)) {
          return number;
        }
        return (number > 0 ? 1 : -1) * Math.floor(Math.abs(number));
      },
          maxSafeInteger = Math.pow(2, 53) - 1,
          toLength = function toLength(value) {
        var len = toInteger(value);
        return Math.min(Math.max(len, 0), maxSafeInteger);
      };
      return function from(arrayLike) {
        var C = this,
            items = Object(arrayLike);
        if (arrayLike == null) {
          throw new TypeError("Array.from must use an Array like object - null ou undefined can't be used");
        }
        var mapFn = arguments.length > 1 ? arguments[1] : void undefined,
            T = void 0;
        if (typeof mapFn !== 'undefined') {
          if (!isCallable(mapFn)) {
            throw new TypeError('Array.from: when used, second argument must be a function');
          }
          if (arguments.length > 2) {
            T = arguments[2];
          }
        }
        var len = toLength(items.length),
            A = isCallable(C) ? Object(new C(len)) : new Array(len),
            k = 0,
            kValue = void 0;
        while (k < len) {
          kValue = items[k];
          if (mapFn) {
            A[k] = typeof T === 'undefined' ? mapFn(kValue, k) : mapFn.call(T, kValue, k);
          } else {
            A[k] = kValue;
          }
          k += 1;
        }
        A.length = len;
        return A;
      };
    }();
  }
}();

return ArrayFromPolyfill;

})));
